#include "MeshLoader.h"

std::vector<Node>& MeshLoader::GetNodes() {
	return nodes;
}

std::vector<FiniteElement>& MeshLoader::GetFE() {
	return finiteElements;
}

std::vector<BoundaryFiniteElement>& MeshLoader::GetBFE() {
	return boundaryFiniteElements;
}

std::vector<int> MeshLoader::findFE(size_t id_1, size_t id_2, size_t id_3) {
	std::vector<int> res;
	auto iter = finiteElements.begin();
	auto end_iter = finiteElements.end();

	while (iter != end_iter) {
		iter = std::find_if(iter, end_iter,
			[id_1, id_2, id_3](const FiniteElement& elem) {
				return std::find(elem.material.begin(), elem.material.end(), id_1) != elem.material.end()
					&& std::find(elem.material.begin(), elem.material.end(), id_2) != elem.material.end()
					&& std::find(elem.material.begin(), elem.material.end(), id_3) != elem.material.end();
			});
		if (iter != end_iter) {
			res.push_back(iter->ID);
			iter++;
		}
	}
	return res;
}

std::vector<FiniteElement> MeshLoader::findEdge(size_t id_1, size_t id_2) {
	std::vector<FiniteElement> res;
	auto iter = finiteElements.begin();
	auto end_iter = finiteElements.end();

	while (iter != end_iter) {
		iter = std::find_if(iter, end_iter, [id_1, id_2](FiniteElement& elem) {
			return std::find(elem.material.begin(), elem.material.end(), id_1) != elem.material.end()
				&& std::find(elem.material.begin(), elem.material.end(), id_2) != elem.material.end();
			});
		if (iter != end_iter) {
			res.push_back(*iter);
			iter++;
		}
	}
	return res;
}

std::vector<size_t> MeshLoader::vertex_node_by_id(size_t id) {
	std::set<Node> res;
	auto iter = boundaryFiniteElements.begin();
	while (iter != boundaryFiniteElements.end()) {
		iter = std::find_if(iter, boundaryFiniteElements.end(), [id](const BoundaryFiniteElement& s) { return s.ID_surface == id; });
		if (iter != boundaryFiniteElements.end())
		{
			for (const auto& elem : iter->node_boundary) {
				res.insert(get_node_by_id(id));
			}
			++iter;
		}
	}
	return std::vector<size_t>(res.begin(), res.end());
}

std::vector<Node> MeshLoader::get_material_id(size_t id) {
	std::list<Node> res;
	auto elem_by_material = nodes.begin();
	auto last_elem = nodes.end();
	while (elem_by_material != last_elem) {

		elem_by_material = std::find_if(elem_by_material, last_elem, [id](const Node& v) { return v.ID_node == id; });

		if (elem_by_material != last_elem) {
			res.push_back(*elem_by_material);
			++elem_by_material;
		}
	}
	return std::vector<Node>(res.begin(), res.end());

}

std::vector<std::set<size_t>> MeshLoader::get_close_nodes() {
	std::vector<std::set<size_t>> res(nodes.size());
	for (const auto& iter_FiniteElement : boundaryFiniteElements)
	{
		for (auto& iter_nodes : iter_FiniteElement.node_boundary)
		{
			res[iter_nodes - 1].insert(iter_FiniteElement.node_boundary.begin(), iter_FiniteElement.node_boundary.end());
			res[iter_nodes - 1].erase(iter_nodes);
		}
	}
	return res;
}

std::vector<BoundaryFiniteElement> MeshLoader::get_BoundaryFinite_id(size_t id) {
	std::list<BoundaryFiniteElement> res;
	auto surface_by_ID = boundaryFiniteElements.begin();
	auto last_elem = boundaryFiniteElements.end();
	while (surface_by_ID != last_elem) {
		surface_by_ID = std::find_if(surface_by_ID, last_elem, [id](const BoundaryFiniteElement& v) { return v.ID_surface == id; });
		if (surface_by_ID != last_elem) {
			res.push_back(*surface_by_ID);
			++surface_by_ID;
		}
	}
	return std::vector<BoundaryFiniteElement>(res.begin(), res.end());

}

Node MeshLoader::get_node(size_t id) {
	nodes.at(id);
}

Node& MeshLoader::get_node_by_id(size_t id) {
	return nodes[id];
}

void MeshLoader::Print() {
	cout << &GetNodes() << endl << &GetFE() << endl << &GetBFE << endl;
}

void MeshLoader::insert_node() {
	std::unordered_set<Edge, HashEdge> edge;
	for (auto& elem : boundaryFiniteElements)
	{
		std::vector<int> iter(elem.ID_boundary);
		for (auto i = iter.begin(); i != iter.end(); i++)
		{
			for (auto j = i + 1; j != iter.end(); j++)
			{
				Edge current_edge(*i, *j);
				auto find_edge = edge.find(current_edge);
				if (find_edge == edge.end()) {
					Node left_node = get_node_by_id(elem.node_boundary[*i]); 
					Node right_node = get_node_by_id(elem.node_boundary[*j]); 
					std::vector<double> x = { (left_node.coord[0] + right_node.coord[0]) / 2,
											 (left_node.coord[0] + right_node.coord[0]) / 2,
											 (left_node.coord[0] + right_node.coord[0]) / 2 };
					Node center_node(nodes.size() + 1, x, false);
					current_edge.self_id = nodes.size() + 1;
					edge.insert(current_edge);
					nodes.push_back(std::move(center_node));
					elem.node_boundary.emplace_back(current_edge.self_id);
				}
				else {
					elem.node_boundary.emplace_back(find_edge->self_id);
				}

			}
		}
	}
	for (auto& it : boundaryFiniteElements)
	{
		std::vector<size_t> current_iter(it.node_boundary);
		for (auto i = current_iter.begin(); i != current_iter.end() - 1; i++)
		{
			for (auto j = i + 1; j != current_iter.end(); j++)
			{
				Edge search_edge(*i, *j);
				auto search_point = edge.find(search_edge);
				if (search_point != edge.end())
				{
					it.node_boundary.emplace_back(search_point->self_id);
				}
			}
		}
	}
}

