#include "Structure.h"

std::ostream& operator << (std::ostream& out, const Node& elem) {
	out << elem.ID_node << " | ";
	for (auto& tmp : elem.coord)
		out << tmp << " ";
	out << " | " << elem.flag << " | " << std::endl;
	return out;
}

std::ostream& operator<<(std::ostream& out, const FiniteElement& elem) {
	out << elem.ID << " | " << elem.ID_material << " | ";
	for (auto& tmp : elem.material)
		out << tmp << " ";
	out << " | " << std::endl;
	return out;
}

std::ostream& operator<< (std::ostream& out, const BoundaryFiniteElement& elem) {
	out << elem.ID_boundary << " | " << elem.ID_surface << " | ";
	for (auto& tmp : elem.node_boundary)
		out << tmp << " ";
	out << " | " << std::endl;
	return out;
}

bool Edge::operator == (const Edge& p) const {
	return (head_id == p.head_id && tail_id == p.tail_id) ||
		(head_id == p.tail_id && tail_id == p.head_id);
}



void HashEdge::hash_combine(size_t& seed, const size_t& val) const {
	seed ^= std::hash<int>()(val) + 0x9e3779b9 + (seed << 6) + (seed >> 2);
}

size_t HashEdge::operator()(const Edge& p) const {
	size_t seed = 0;
	hash_combine(seed, p.head_id);
	hash_combine(seed, p.tail_id);
	hash_combine(seed, p.self_id);

	return seed;
}